%{ open Ast %}

%token SEMI LPAREN RPAREN LBRACE RBRACE COMMA 
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ 
%token RETURN IF ELSE FOR WHILE 
%token INT POINT LINE CURVE ELLIPSE STRING VOID BOOLEAN
%token <int> INTLITERAL
%token <string> STRINGLITERAL
%token <string> ID
%token MPOINT MLINE MELLIPSE MCURVE
%token ASSIGN
%token PRINT
%token DRAW GETX GETY
%token EOF

%nonassoc NOELSE 
%nonassoc ELSE 
%right ASSIGN
%left EQ NEQ
%left LT GT LEQ GEQ 
%left PLUS MINUS 
%left TIMES DIVIDE

%start program
%type <Ast.program> program

%%

program:
    program_r { List.rev $1 }

program_r:
   /* nothing */ { [] }
  | program_r fdecl { FuncDef($2)::$1 }
  | program_r vdecl { GlobalVar($2)::$1 }
 
 
/* List of all return types */
Type:
      INT {Int}
    | POINT {Point}
    | LINE {Line}
    | ELLIPSE {Ellipse}
    | CURVE {Curve}
    /*| LAYER {Layer}*/
    | STRING {String}  /* Need void here?*/
    | BOOLEAN {Boolean}
    | VOID {Void}
	
fdecl:
    Type ID LPAREN formals_opt RPAREN block_stmt
      { { 
         return = $1;
         fname = $2;
         formals = $4;
         body = $6; } }
   

formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formal_list:
    Type ID { [VarSig($1,$2)] }
  | formal_list COMMA Type ID { VarSig($3,$4) :: $1 }

vdecl:
     Type ID SEMI { NullInit($1,$2) }   /*default initialization*/
   | Type ID ASSIGN expr SEMI { VarInit($1,$2,$4) }   /*explicit initialization*/

block_stmt:
    LBRACE stmt_list RBRACE { Block (List.rev $2) }
    
stmt_list:
    /* nothing */  { [] }
  | stmt_list stmt { $2 :: $1 }

stmt:
    expr SEMI { Expr($1) }
  | RETURN expr SEMI { Return($2) }
  | LBRACE stmt_list RBRACE { Block(List.rev $2) }
  | IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, NoElseMark) }
  | IF LPAREN expr RPAREN stmt ELSE stmt    { If($3, $5, $7) }
  | FOR LPAREN expr SEMI expr SEMI expr RPAREN block_stmt
     { For($3, $5, $7, $9) }
  | WHILE LPAREN expr RPAREN stmt { While($3, $5) }
  | PRINT expr SEMI { Print($2)}
  | expr DRAW SEMI	{ Draw($1)}
  | vdecl {VarDecl($1)}


expr:
  | INTLITERAL          { IntLiteral($1) }
  | STRINGLITERAL	{ StringLiteral($1) }
  | ID              	 { Id($1) }
  | MPOINT LPAREN expr COMMA expr RPAREN { MPoint($3, $5)}
  | MLINE LPAREN expr COMMA expr COMMA expr COMMA expr RPAREN { MLine($3, $5, $7, $9)}
  | MELLIPSE LPAREN expr COMMA expr COMMA expr COMMA expr RPAREN { MEllipse($3, $5, $7, $9)}
  | MCURVE LPAREN expr COMMA expr COMMA expr COMMA expr COMMA expr COMMA expr RPAREN { MCurve($3, $5, $7, $9, $11, $13)}
  | expr GETX {GetX($1)}
  | expr GETY {GetY($1)}
  | expr PLUS   expr 	{ Binop($1, Add,   $3) }
  | expr MINUS  expr 	{ Binop($1, Sub,   $3) }
  | expr TIMES  expr 	{ Binop($1, Mult,  $3) }
  | expr DIVIDE expr 	{ Binop($1, Div,   $3) }
  | MINUS  expr      	{ Binop(IntLiteral(0),  Sub,   $2) }
  | expr EQ     expr 	{ Binop($1, Equal, $3) }
  | expr NEQ    expr 	{ Binop($1, Neq,   $3) }
  | expr LT     expr 	{ Binop($1, Less,  $3) }
  | expr LEQ    expr 	{ Binop($1, Leq,   $3) }
  | expr GT     expr 	{ Binop($1, Greater,  $3) }
  | expr GEQ    expr 	{ Binop($1, Geq,   $3) }
  | expr ASSIGN expr   	{ Assign($1, $3) }
  | ID LPAREN actuals_opt RPAREN 
  			{ Call($1, $3) }    /* function call*/
  | LPAREN expr RPAREN 	{ $2 }

actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }



